@@ -814,6 +814,8 @@ class ElfBuilder final {
         return InstructionSet::kThumb2;
       case EM_AARCH64:
         return InstructionSet::kArm64;
+      case EM_RISCV:
+        return InstructionSet::kRiscv64;
       case EM_386:
         return InstructionSet::kX86;
       case EM_X86_64:
@@ -839,6 +841,11 @@ class ElfBuilder final {
         elf_header.e_flags = 0;
         break;
       }
+      case InstructionSet::kRiscv64: {
+        elf_header.e_machine = EM_RISCV;
+        elf_header.e_flags = EF_RISCV_RVC | EF_RISCV_FLOAT_ABI_DOUBLE;
+        break;
+      }
       case InstructionSet::kX86: {
         elf_header.e_machine = EM_386;
         elf_header.e_flags = 0;
