@@ -50,7 +50,7 @@ class ImageSpaceTest : public CommonRuntimeTest {
 };
 
 TEST_F(ImageSpaceTest, StringDeduplication) {
-  const char* const kBaseNames[] = { "Extension1", "Extension2" };
+  const char* const kBaseNames[] = {"Extension1", "Extension2"};
 
   ScratchDir scratch;
   const std::string& scratch_dir = scratch.GetPath();
@@ -77,7 +77,7 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
   std::vector<std::string> extension_image_locations;
   for (const char* base_name : kBaseNames) {
     std::string jar_name = GetTestDexFileName(base_name);
-    ArrayRef<const std::string> dex_files(&jar_name, /*size=*/ 1u);
+    ArrayRef<const std::string> dex_files(&jar_name, /*size=*/1u);
     ScratchFile profile_file;
     GenerateBootProfile(dex_files, profile_file.GetFile());
     std::vector<std::string> extra_args = {
@@ -94,8 +94,8 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
     ASSERT_TRUE(success) << error_msg;
     bcp.push_back(jar_name);
     bcp_locations.push_back(jar_name);
-    extension_image_locations.push_back(
-        scratch_dir + prefix + '-' + GetFilenameBase(jar_name) + ".art");
+    extension_image_locations.push_back(scratch_dir + prefix + '-' + GetFilenameBase(jar_name) +
+                                        ".art");
   }
 
   // Also compile the second extension as an app with app image.
@@ -104,26 +104,27 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
   std::string app_odex_name = scratch_dir + app_base_name + ".odex";
   std::string app_image_name = scratch_dir + app_base_name + ".art";
   {
-    ArrayRef<const std::string> dex_files(&app_jar_name, /*size=*/ 1u);
+    ArrayRef<const std::string> dex_files(&app_jar_name, /*size=*/1u);
     ScratchFile profile_file;
     GenerateProfile(dex_files, profile_file.GetFile());
     std::vector<std::string> argv;
     std::string error_msg;
-    bool success = StartDex2OatCommandLine(&argv, &error_msg, /*use_runtime_bcp_and_image=*/ false);
+    bool success = StartDex2OatCommandLine(&argv, &error_msg, /*use_runtime_bcp_and_image=*/false);
     ASSERT_TRUE(success) << error_msg;
-    argv.insert(argv.end(), {
-        "--profile-file=" + profile_file.GetFilename(),
-        "--runtime-arg",
-        "-Xbootclasspath:" + base_bcp_string,
-        "--runtime-arg",
-        "-Xbootclasspath-locations:" + base_bcp_locations_string,
-        "--boot-image=" + base_image_location,
-        "--dex-file=" + app_jar_name,
-        "--dex-location=" + app_jar_name,
-        "--oat-file=" + app_odex_name,
-        "--app-image-file=" + app_image_name,
-        "--initialize-app-image-classes=true",
-    });
+    argv.insert(argv.end(),
+                {
+                    "--profile-file=" + profile_file.GetFilename(),
+                    "--runtime-arg",
+                    "-Xbootclasspath:" + base_bcp_string,
+                    "--runtime-arg",
+                    "-Xbootclasspath-locations:" + base_bcp_locations_string,
+                    "--boot-image=" + base_image_location,
+                    "--dex-file=" + app_jar_name,
+                    "--dex-location=" + app_jar_name,
+                    "--oat-file=" + app_odex_name,
+                    "--app-image-file=" + app_image_name,
+                    "--initialize-app-image-classes=true",
+                });
     success = RunDex2Oat(argv, &error_msg);
     ASSERT_TRUE(success) << error_msg;
   }
@@ -136,15 +137,16 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
     extra_reservation = MemMap::Invalid();
     return ImageSpace::LoadBootImage(bcp,
                                      bcp_locations,
-                                     /*boot_class_path_fds=*/ std::vector<int>(),
-                                     /*boot_class_path_image_fds=*/ std::vector<int>(),
-                                     /*boot_class_path_vdex_fds=*/ std::vector<int>(),
-                                     /*boot_class_path_oat_fds=*/ std::vector<int>(),
+                                     /*boot_class_path_fds=*/std::vector<int>(),
+                                     /*boot_class_path_image_fds=*/std::vector<int>(),
+                                     /*boot_class_path_vdex_fds=*/std::vector<int>(),
+                                     /*boot_class_path_oat_fds=*/std::vector<int>(),
                                      full_image_locations,
                                      kRuntimeISA,
-                                     /*relocate=*/ false,
-                                     /*executable=*/ true,
-                                     /*extra_reservation_size=*/ 0u,
+                                     /*relocate=*/false,
+                                     /*executable=*/true,
+                                     /*extra_reservation_size=*/0u,
+                                     /*allow_in_memory_compilation=*/false,
                                      &boot_image_spaces,
                                      &extra_reservation);
   };
@@ -153,13 +155,13 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
   size_t test_string_length = std::size(test_string) - 1u;  // Equals UTF-16 length.
   uint32_t hash = InternTable::Utf8String::Hash(test_string_length, test_string);
   InternTable::Utf8String utf8_test_string(test_string_length, test_string);
-  auto contains_test_string = [utf8_test_string, hash](ImageSpace* space)
-      REQUIRES_SHARED(Locks::mutator_lock_) {
+  auto contains_test_string = [utf8_test_string,
+                               hash](ImageSpace* space) REQUIRES_SHARED(Locks::mutator_lock_) {
     const ImageHeader& image_header = space->GetImageHeader();
     if (image_header.GetInternedStringsSection().Size() != 0u) {
       const uint8_t* data = space->Begin() + image_header.GetInternedStringsSection().Offset();
       size_t read_count;
-      InternTable::UnorderedSet temp_set(data, /*make_copy_of_data=*/ false, &read_count);
+      InternTable::UnorderedSet temp_set(data, /*make_copy_of_data=*/false, &read_count);
       return temp_set.FindWithHash(utf8_test_string, hash) != temp_set.end();
     } else {
       return false;
@@ -170,8 +172,7 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
   ScopedObjectAccess soa(Thread::Current());
   ASSERT_EQ(2u, extension_image_locations.size());
   full_image_locations = {
-    base_image_location, extension_image_locations[0], extension_image_locations[1]
-  };
+      base_image_location, extension_image_locations[0], extension_image_locations[1]};
   bool success = load_boot_image();
   ASSERT_TRUE(success);
   ASSERT_EQ(bcp.size(), boot_image_spaces.size());
@@ -183,8 +184,7 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
   std::swap(bcp[bcp.size() - 2u], bcp[bcp.size() - 1u]);
   std::swap(bcp_locations[bcp_locations.size() - 2u], bcp_locations[bcp_locations.size() - 1u]);
   full_image_locations = {
-    base_image_location, extension_image_locations[1], extension_image_locations[0]
-  };
+      base_image_location, extension_image_locations[1], extension_image_locations[0]};
   success = load_boot_image();
   ASSERT_TRUE(success);
   ASSERT_EQ(bcp.size(), boot_image_spaces.size());
@@ -203,21 +203,21 @@ TEST_F(ImageSpaceTest, StringDeduplication) {
 
   // Load the app odex file and app image.
   std::string error_msg;
-  std::unique_ptr<OatFile> odex_file(OatFile::Open(/*zip_fd=*/ -1,
-                                                   app_odex_name.c_str(),
-                                                   app_odex_name.c_str(),
-                                                   /*executable=*/ false,
-                                                   /*low_4gb=*/ false,
+  std::unique_ptr<OatFile> odex_file(OatFile::Open(/*zip_fd=*/-1,
+                                                   app_odex_name,
+                                                   app_odex_name,
+                                                   /*executable=*/false,
+                                                   /*low_4gb=*/false,
                                                    app_jar_name,
                                                    &error_msg));
   ASSERT_TRUE(odex_file != nullptr) << error_msg;
   std::vector<ImageSpace*> non_owning_boot_image_spaces =
       MakeNonOwningPointerVector(boot_image_spaces);
-  std::unique_ptr<ImageSpace> app_image_space = ImageSpace::CreateFromAppImage(
-      app_image_name.c_str(),
-      odex_file.get(),
-      ArrayRef<ImageSpace* const>(non_owning_boot_image_spaces),
-      &error_msg);
+  std::unique_ptr<ImageSpace> app_image_space =
+      ImageSpace::CreateFromAppImage(app_image_name.c_str(),
+                                     odex_file.get(),
+                                     ArrayRef<ImageSpace* const>(non_owning_boot_image_spaces),
+                                     &error_msg);
   ASSERT_TRUE(app_image_space != nullptr) << error_msg;
 
   // The string in the app image should be replaced and removed from interned string section.
@@ -242,25 +242,25 @@ TEST_F(DexoptTest, ValidateOatFile) {
   args.push_back("--oat-file=" + oat_location);
   ASSERT_TRUE(Dex2Oat(args, &error_msg)) << error_msg;
 
-  std::unique_ptr<OatFile> oat(OatFile::Open(/*zip_fd=*/ -1,
-                                             oat_location.c_str(),
-                                             oat_location.c_str(),
-                                             /*executable=*/ false,
-                                             /*low_4gb=*/ false,
+  std::unique_ptr<OatFile> oat(OatFile::Open(/*zip_fd=*/-1,
+                                             oat_location,
+                                             oat_location,
+                                             /*executable=*/false,
+                                             /*low_4gb=*/false,
                                              &error_msg));
   ASSERT_TRUE(oat != nullptr) << error_msg;
 
   {
     // Test opening the oat file also with explicit dex filenames.
-    std::vector<std::string> dex_filenames{ dex1, multidex1, dex2 };
-    std::unique_ptr<OatFile> oat2(OatFile::Open(/*zip_fd=*/ -1,
-                                                oat_location.c_str(),
-                                                oat_location.c_str(),
-                                                /*executable=*/ false,
-                                                /*low_4gb=*/ false,
+    std::vector<std::string> dex_filenames{dex1, multidex1, dex2};
+    std::unique_ptr<OatFile> oat2(OatFile::Open(/*zip_fd=*/-1,
+                                                oat_location,
+                                                oat_location,
+                                                /*executable=*/false,
+                                                /*low_4gb=*/false,
                                                 ArrayRef<const std::string>(dex_filenames),
-                                                /*dex_fds=*/ ArrayRef<const int>(),
-                                                /*reservation=*/ nullptr,
+                                                /*dex_fds=*/ArrayRef<const int>(),
+                                                /*reservation=*/nullptr,
                                                 &error_msg));
     ASSERT_TRUE(oat2 != nullptr) << error_msg;
   }
@@ -321,56 +321,6 @@ TEST_F(DexoptTest, ValidateOatFile) {
   EXPECT_FALSE(ImageSpace::ValidateOatFile(*oat, &error_msg));
 }
 
-TEST_F(DexoptTest, Checksums) {
-  Runtime* runtime = Runtime::Current();
-  ASSERT_TRUE(runtime != nullptr);
-  ASSERT_FALSE(runtime->GetHeap()->GetBootImageSpaces().empty());
-
-  std::vector<std::string> bcp = runtime->GetBootClassPath();
-  std::vector<std::string> bcp_locations = runtime->GetBootClassPathLocations();
-  std::vector<const DexFile*> dex_files = runtime->GetClassLinker()->GetBootClassPath();
-
-  std::string error_msg;
-  auto create_and_verify = [&]() {
-    std::string checksums = gc::space::ImageSpace::GetBootClassPathChecksums(
-        ArrayRef<gc::space::ImageSpace* const>(runtime->GetHeap()->GetBootImageSpaces()),
-        ArrayRef<const DexFile* const>(dex_files));
-    return gc::space::ImageSpace::VerifyBootClassPathChecksums(
-        checksums,
-        android::base::Join(bcp_locations, ':'),
-        ArrayRef<const std::string>(runtime->GetImageLocations()),
-        ArrayRef<const std::string>(bcp_locations),
-        ArrayRef<const std::string>(bcp),
-        /*boot_class_path_fds=*/ ArrayRef<const int>(),
-        kRuntimeISA,
-        &error_msg);
-  };
-
-  ASSERT_TRUE(create_and_verify()) << error_msg;
-
-  std::vector<std::unique_ptr<const DexFile>> opened_dex_files;
-  for (const std::string& src : { GetDexSrc1(), GetDexSrc2() }) {
-    std::vector<std::unique_ptr<const DexFile>> new_dex_files;
-    const ArtDexFileLoader dex_file_loader;
-    ASSERT_TRUE(dex_file_loader.Open(src.c_str(),
-                                     src,
-                                     /*verify=*/ true,
-                                     /*verify_checksum=*/ false,
-                                     &error_msg,
-                                     &new_dex_files))
-        << error_msg;
-
-    bcp.push_back(src);
-    bcp_locations.push_back(src);
-    for (std::unique_ptr<const DexFile>& df : new_dex_files) {
-      dex_files.push_back(df.get());
-      opened_dex_files.push_back(std::move(df));
-    }
-
-    ASSERT_TRUE(create_and_verify()) << error_msg;
-  }
-}
-
 template <bool kImage, bool kRelocate>
 class ImageSpaceLoadingTest : public CommonRuntimeTest {
  protected:
@@ -380,6 +330,7 @@ class ImageSpaceLoadingTest : public CommonRuntimeTest {
     options->emplace_back(android::base::StringPrintf("-Ximage:%s", image_location.c_str()),
                           nullptr);
     options->emplace_back(kRelocate ? "-Xrelocate" : "-Xnorelocate", nullptr);
+    options->emplace_back("-Xallowinmemorycompilation", nullptr);
 
     // We want to test the relocation behavior of ImageSpace. As such, don't pretend we're a
     // compiler.
