@@ -20,7 +20,6 @@
 #include "android-base/macros.h"
 #include "instrumentation.h"
 #include "interpreter.h"
-#include "interpreter_intrinsics.h"
 #include "transaction.h"
 
 #include <math.h>
@@ -71,7 +70,6 @@ namespace interpreter {
 void ThrowNullPointerExceptionFromInterpreter()
     REQUIRES_SHARED(Locks::mutator_lock_);
 
-template <bool kMonitorCounting>
 static inline void DoMonitorEnter(Thread* self, ShadowFrame* frame, ObjPtr<mirror::Object> ref)
     NO_THREAD_SAFETY_ANALYSIS
     REQUIRES(!Roles::uninterruptible_) {
@@ -85,28 +83,29 @@ static inline void DoMonitorEnter(Thread* self, ShadowFrame* frame, ObjPtr<mirro
     DCHECK(unlocked);
     return;
   }
-  if (kMonitorCounting && frame->GetMethod()->MustCountLocks()) {
+  if (frame->GetMethod()->MustCountLocks()) {
+    DCHECK(!frame->GetMethod()->SkipAccessChecks());
     frame->GetLockCountData().AddMonitor(self, h_ref.Get());
   }
 }
 
-template <bool kMonitorCounting>
 static inline void DoMonitorExit(Thread* self, ShadowFrame* frame, ObjPtr<mirror::Object> ref)
     NO_THREAD_SAFETY_ANALYSIS
     REQUIRES(!Roles::uninterruptible_) {
   StackHandleScope<1> hs(self);
   Handle<mirror::Object> h_ref(hs.NewHandle(ref));
   h_ref->MonitorExit(self);
-  if (kMonitorCounting && frame->GetMethod()->MustCountLocks()) {
+  if (frame->GetMethod()->MustCountLocks()) {
+    DCHECK(!frame->GetMethod()->SkipAccessChecks());
     frame->GetLockCountData().RemoveMonitorOrThrow(self, h_ref.Get());
   }
 }
 
-template <bool kMonitorCounting>
 static inline bool DoMonitorCheckOnExit(Thread* self, ShadowFrame* frame)
     NO_THREAD_SAFETY_ANALYSIS
     REQUIRES(!Roles::uninterruptible_) {
-  if (kMonitorCounting && frame->GetMethod()->MustCountLocks()) {
+  if (frame->GetMethod()->MustCountLocks()) {
+    DCHECK(!frame->GetMethod()->SkipAccessChecks());
     return frame->GetLockCountData().CheckAllMonitorsReleasedOrThrow(self);
   }
   return true;
@@ -125,9 +124,14 @@ void RecordArrayElementsInTransaction(ObjPtr<mirror::Array> array, int32_t count
 // Invokes the given method. This is part of the invocation support and is used by DoInvoke,
 // DoFastInvoke and DoInvokeVirtualQuick functions.
 // Returns true on success, otherwise throws an exception and returns false.
-template<bool is_range, bool do_assignability_check>
-bool DoCall(ArtMethod* called_method, Thread* self, ShadowFrame& shadow_frame,
-            const Instruction* inst, uint16_t inst_data, JValue* result);
+template<bool is_range>
+bool DoCall(ArtMethod* called_method,
+            Thread* self,
+            ShadowFrame& shadow_frame,
+            const Instruction* inst,
+            uint16_t inst_data,
+            bool string_init,
+            JValue* result);
 
 // Called by the switch interpreter to know if we can stay in it.
 bool ShouldStayInSwitchInterpreter(ArtMethod* method)
@@ -153,54 +157,16 @@ NeedsMethodExitEvent(const instrumentation::Instrumentation* ins)
   return ins->HasMethodExitListeners() || ins->HasWatchedFramePopListeners();
 }
 
-// NO_INLINE so we won't bloat the interpreter with this very cold lock-release code.
-template <bool kMonitorCounting>
-static NO_INLINE void UnlockHeldMonitors(Thread* self, ShadowFrame* shadow_frame)
-    REQUIRES_SHARED(Locks::mutator_lock_) {
-  DCHECK(shadow_frame->GetForcePopFrame() ||
-         Runtime::Current()->IsTransactionAborted());
-  // Unlock all monitors.
-  if (kMonitorCounting && shadow_frame->GetMethod()->MustCountLocks()) {
-    // Get the monitors from the shadow-frame monitor-count data.
-    shadow_frame->GetLockCountData().VisitMonitors(
-      [&](mirror::Object** obj) REQUIRES_SHARED(Locks::mutator_lock_) {
-        // Since we don't use the 'obj' pointer after the DoMonitorExit everything should be fine
-        // WRT suspension.
-        DoMonitorExit<kMonitorCounting>(self, shadow_frame, *obj);
-      });
-  } else {
-    std::vector<verifier::MethodVerifier::DexLockInfo> locks;
-    verifier::MethodVerifier::FindLocksAtDexPc(shadow_frame->GetMethod(),
-                                                shadow_frame->GetDexPC(),
-                                                &locks,
-                                                Runtime::Current()->GetTargetSdkVersion());
-    for (const auto& reg : locks) {
-      if (UNLIKELY(reg.dex_registers.empty())) {
-        LOG(ERROR) << "Unable to determine reference locked by "
-                    << shadow_frame->GetMethod()->PrettyMethod() << " at pc "
-                    << shadow_frame->GetDexPC();
-      } else {
-        DoMonitorExit<kMonitorCounting>(
-            self, shadow_frame, shadow_frame->GetVRegReference(*reg.dex_registers.begin()));
-      }
-    }
-  }
-}
+COLD_ATTR void UnlockHeldMonitors(Thread* self, ShadowFrame* shadow_frame)
+    REQUIRES_SHARED(Locks::mutator_lock_);
 
-enum class MonitorState {
-  kNoMonitorsLocked,
-  kCountingMonitors,
-  kNormalMonitors,
-};
-
-template<MonitorState kMonitorState>
 static inline ALWAYS_INLINE void PerformNonStandardReturn(
       Thread* self,
       ShadowFrame& frame,
       JValue& result,
       const instrumentation::Instrumentation* instrumentation,
-      uint16_t num_dex_inst) REQUIRES_SHARED(Locks::mutator_lock_) {
-  static constexpr bool kMonitorCounting = (kMonitorState == MonitorState::kCountingMonitors);
+      uint16_t num_dex_inst,
+      bool unlock_monitors = true) REQUIRES_SHARED(Locks::mutator_lock_) {
   ObjPtr<mirror::Object> thiz(frame.GetThisObject(num_dex_inst));
   StackHandleScope<1u> hs(self);
   if (UNLIKELY(self->IsExceptionPending())) {
@@ -208,10 +174,10 @@ static inline ALWAYS_INLINE void PerformNonStandardReturn(
                  << self->GetException()->Dump();
     self->ClearException();
   }
-  if (kMonitorState != MonitorState::kNoMonitorsLocked) {
-    UnlockHeldMonitors<kMonitorCounting>(self, &frame);
+  if (unlock_monitors) {
+    UnlockHeldMonitors(self, &frame);
+    DoMonitorCheckOnExit(self, &frame);
   }
-  DoMonitorCheckOnExit<kMonitorCounting>(self, &frame);
   result = JValue();
   if (UNLIKELY(NeedsMethodExitEvent(instrumentation))) {
     SendMethodExitEvents(self, instrumentation, frame, frame.GetMethod(), result);
@@ -220,7 +186,7 @@ static inline ALWAYS_INLINE void PerformNonStandardReturn(
 
 // Handles all invoke-XXX/range instructions except for invoke-polymorphic[/range].
 // Returns true on success, otherwise throws an exception and returns false.
-template<InvokeType type, bool is_range, bool do_access_check, bool is_mterp>
+template<InvokeType type, bool is_range>
 static ALWAYS_INLINE bool DoInvoke(Thread* self,
                                    ShadowFrame& shadow_frame,
                                    const Instruction* inst,
@@ -231,63 +197,20 @@ static ALWAYS_INLINE bool DoInvoke(Thread* self,
   if (UNLIKELY(self->ObserveAsyncException())) {
     return false;
   }
-  const uint32_t method_idx = (is_range) ? inst->VRegB_3rc() : inst->VRegB_35c();
-  const uint32_t vregC = (is_range) ? inst->VRegC_3rc() : inst->VRegC_35c();
+  const uint32_t vregC = is_range ? inst->VRegC_3rc() : inst->VRegC_35c();
+  ObjPtr<mirror::Object> obj = type == kStatic ? nullptr : shadow_frame.GetVRegReference(vregC);
   ArtMethod* sf_method = shadow_frame.GetMethod();
-
-  // Try to find the method in small thread-local cache first (only used when
-  // nterp is not used as mterp and nterp use the cache in an incompatible way).
-  InterpreterCache* tls_cache = self->GetInterpreterCache();
-  size_t tls_value;
-  ArtMethod* resolved_method;
-  if (!IsNterpSupported() && LIKELY(tls_cache->Get(self, inst, &tls_value))) {
-    resolved_method = reinterpret_cast<ArtMethod*>(tls_value);
-  } else {
-    ClassLinker* const class_linker = Runtime::Current()->GetClassLinker();
-    constexpr ClassLinker::ResolveMode resolve_mode =
-        do_access_check ? ClassLinker::ResolveMode::kCheckICCEAndIAE
-                        : ClassLinker::ResolveMode::kNoChecks;
-    resolved_method = class_linker->ResolveMethod<resolve_mode>(self, method_idx, sf_method, type);
-    if (UNLIKELY(resolved_method == nullptr)) {
-      CHECK(self->IsExceptionPending());
-      result->SetJ(0);
-      return false;
-    }
-    if (!IsNterpSupported()) {
-      tls_cache->Set(self, inst, reinterpret_cast<size_t>(resolved_method));
-    }
-  }
-
-  // Null pointer check and virtual method resolution.
-  ObjPtr<mirror::Object> receiver =
-      (type == kStatic) ? nullptr : shadow_frame.GetVRegReference(vregC);
-  ArtMethod* called_method;
-  called_method = FindMethodToCall<type, do_access_check>(
-      method_idx, resolved_method, &receiver, sf_method, self);
-  if (UNLIKELY(called_method == nullptr)) {
-    CHECK(self->IsExceptionPending());
-    result->SetJ(0);
-    return false;
-  }
-  if (UNLIKELY(!called_method->IsInvokable())) {
-    called_method->ThrowInvocationTimeError();
+  bool string_init = false;
+  ArtMethod* called_method = FindMethodToCall<type>(
+      self, sf_method, &obj, *inst, /* only_lookup_tls_cache= */ false, &string_init);
+  if (called_method == nullptr) {
+    DCHECK(self->IsExceptionPending());
     result->SetJ(0);
     return false;
   }
 
-  jit::Jit* jit = Runtime::Current()->GetJit();
-  if (is_mterp && !is_range && called_method->IsIntrinsic()) {
-    if (MterpHandleIntrinsic(&shadow_frame, called_method, inst, inst_data,
-                             shadow_frame.GetResultRegister())) {
-      if (jit != nullptr && sf_method != nullptr) {
-        jit->NotifyInterpreterToCompiledCodeTransition(self, sf_method);
-      }
-      return !self->IsExceptionPending();
-    }
-  }
-
-  return DoCall<is_range, do_access_check>(called_method, self, shadow_frame, inst, inst_data,
-                                           result);
+  return DoCall<is_range>(
+      called_method, self, shadow_frame, inst, inst_data, string_init, result);
 }
 
 static inline ObjPtr<mirror::MethodHandle> ResolveMethodHandle(Thread* self,
@@ -386,24 +309,77 @@ ALWAYS_INLINE static JValue GetFieldValue(const ShadowFrame& shadow_frame, uint3
   return field_value;
 }
 
+extern "C" size_t NterpGetStaticField(Thread* self,
+                                      ArtMethod* caller,
+                                      const uint16_t* dex_pc_ptr,
+                                      size_t resolve_field_type);
+
+extern "C" uint32_t NterpGetInstanceFieldOffset(Thread* self,
+                                                ArtMethod* caller,
+                                                const uint16_t* dex_pc_ptr,
+                                                size_t resolve_field_type);
+
+static inline void GetFieldInfo(Thread* self,
+                                ArtMethod* caller,
+                                const uint16_t* dex_pc_ptr,
+                                bool is_static,
+                                bool resolve_field_type,
+                                ArtField** field,
+                                bool* is_volatile,
+                                MemberOffset* offset) {
+  size_t tls_value = 0u;
+  if (!self->GetInterpreterCache()->Get(self, dex_pc_ptr, &tls_value)) {
+    if (is_static) {
+      tls_value = NterpGetStaticField(self, caller, dex_pc_ptr, resolve_field_type);
+    } else {
+      tls_value = NterpGetInstanceFieldOffset(self, caller, dex_pc_ptr, resolve_field_type);
+    }
+
+    if (self->IsExceptionPending()) {
+      return;
+    }
+  }
+
+  if (is_static) {
+    DCHECK_NE(tls_value, 0u);
+    *is_volatile = ((tls_value & 1) != 0);
+    *field = reinterpret_cast<ArtField*>(tls_value & ~static_cast<size_t>(1u));
+    *offset = (*field)->GetOffset();
+  } else {
+    *is_volatile = (static_cast<int32_t>(tls_value) < 0);
+    *offset = MemberOffset(std::abs(static_cast<int32_t>(tls_value)));
+  }
+}
+
 // Handles iget-XXX and sget-XXX instructions.
 // Returns true on success, otherwise throws an exception and returns false.
-template<FindFieldType find_type, Primitive::Type field_type, bool do_access_check,
+template<FindFieldType find_type,
+         Primitive::Type field_type,
          bool transaction_active = false>
-ALWAYS_INLINE bool DoFieldGet(Thread* self, ShadowFrame& shadow_frame, const Instruction* inst,
+ALWAYS_INLINE bool DoFieldGet(Thread* self,
+                              ShadowFrame& shadow_frame,
+                              const Instruction* inst,
                               uint16_t inst_data) REQUIRES_SHARED(Locks::mutator_lock_) {
   const bool is_static = (find_type == StaticObjectRead) || (find_type == StaticPrimitiveRead);
-  const uint32_t field_idx = is_static ? inst->VRegB_21c() : inst->VRegC_22c();
-  ArtMethod* method = shadow_frame.GetMethod();
-  ArtField* f = FindFieldFromCode<find_type, do_access_check>(
-      field_idx, method, self, Primitive::ComponentSize(field_type));
-  if (UNLIKELY(f == nullptr)) {
-    CHECK(self->IsExceptionPending());
+  bool should_report = Runtime::Current()->GetInstrumentation()->HasFieldReadListeners();
+  ArtField* field = nullptr;
+  MemberOffset offset(0u);
+  bool is_volatile;
+  GetFieldInfo(self,
+               shadow_frame.GetMethod(),
+               reinterpret_cast<const uint16_t*>(inst),
+               is_static,
+               /*resolve_field_type=*/ false,
+               &field,
+               &is_volatile,
+               &offset);
+  if (self->IsExceptionPending()) {
     return false;
   }
+
   ObjPtr<mirror::Object> obj;
   if (is_static) {
-    obj = f->GetDeclaringClass();
+    obj = field->GetDeclaringClass();
     if (transaction_active) {
       if (Runtime::Current()->GetTransaction()->ReadConstraint(obj)) {
         Runtime::Current()->AbortTransactionAndThrowAbortError(self, "Can't read static fields of "
@@ -413,40 +389,57 @@ ALWAYS_INLINE bool DoFieldGet(Thread* self, ShadowFrame& shadow_frame, const Ins
     }
   } else {
     obj = shadow_frame.GetVRegReference(inst->VRegB_22c(inst_data));
-    if (UNLIKELY(obj == nullptr)) {
-      ThrowNullPointerExceptionForFieldAccess(f, method, true);
+    if (should_report || obj == nullptr) {
+      field = ResolveFieldWithAccessChecks(self,
+                                           Runtime::Current()->GetClassLinker(),
+                                           inst->VRegC_22c(),
+                                           shadow_frame.GetMethod(),
+                                           /* is_static= */ false,
+                                           /* is_put= */ false,
+                                           /* resolve_field_type= */ false);
+      if (obj == nullptr) {
+        ThrowNullPointerExceptionForFieldAccess(
+            field, shadow_frame.GetMethod(), /* is_read= */ true);
+        return false;
+      }
+      // Reload in case suspension happened during field resolution.
+      obj = shadow_frame.GetVRegReference(inst->VRegB_22c(inst_data));
+    }
+  }
+
+  uint32_t vregA = is_static ? inst->VRegA_21c(inst_data) : inst->VRegA_22c(inst_data);
+  JValue result;
+  if (should_report) {
+    DCHECK(field != nullptr);
+    if (UNLIKELY(!DoFieldGetCommon<field_type>(self, shadow_frame, obj, field, &result))) {
+      // Instrumentation threw an error!
+      CHECK(self->IsExceptionPending());
       return false;
     }
   }
 
-  JValue result;
-  if (UNLIKELY(!DoFieldGetCommon<field_type>(self, shadow_frame, obj, f, &result))) {
-    // Instrumentation threw an error!
-    CHECK(self->IsExceptionPending());
-    return false;
-  }
-  uint32_t vregA = is_static ? inst->VRegA_21c(inst_data) : inst->VRegA_22c(inst_data);
+#define FIELD_GET(prim, type, jtype, vreg)                                      \
+  case Primitive::kPrim ##prim:                                                 \
+    shadow_frame.SetVReg ##vreg(vregA,                                          \
+        should_report ? result.Get ##jtype()                                    \
+                      : is_volatile ? obj->GetField ## type ## Volatile(offset) \
+                                    : obj->GetField ##type(offset));            \
+    break;
+
   switch (field_type) {
-    case Primitive::kPrimBoolean:
-      shadow_frame.SetVReg(vregA, result.GetZ());
-      break;
-    case Primitive::kPrimByte:
-      shadow_frame.SetVReg(vregA, result.GetB());
-      break;
-    case Primitive::kPrimChar:
-      shadow_frame.SetVReg(vregA, result.GetC());
-      break;
-    case Primitive::kPrimShort:
-      shadow_frame.SetVReg(vregA, result.GetS());
-      break;
-    case Primitive::kPrimInt:
-      shadow_frame.SetVReg(vregA, result.GetI());
-      break;
-    case Primitive::kPrimLong:
-      shadow_frame.SetVRegLong(vregA, result.GetJ());
-      break;
+    FIELD_GET(Boolean, Boolean, Z, )
+    FIELD_GET(Byte, Byte, B, )
+    FIELD_GET(Char, Char, C, )
+    FIELD_GET(Short, Short, S, )
+    FIELD_GET(Int, 32, I, )
+    FIELD_GET(Long, 64, J, Long)
+#undef FIELD_GET
     case Primitive::kPrimNot:
-      shadow_frame.SetVRegReference(vregA, result.GetL());
+      shadow_frame.SetVRegReference(
+          vregA,
+          should_report ? result.GetL()
+                        : is_volatile ? obj->GetFieldObjectVolatile<mirror::Object>(offset)
+                                      : obj->GetFieldObject<mirror::Object>(offset));
       break;
     default:
       LOG(FATAL) << "Unreachable: " << field_type;
@@ -485,36 +478,57 @@ static inline bool CheckWriteValueConstraint(Thread* self, ObjPtr<mirror::Object
 
 // Handles iput-XXX and sput-XXX instructions.
 // Returns true on success, otherwise throws an exception and returns false.
-template<FindFieldType find_type, Primitive::Type field_type, bool do_access_check,
-         bool transaction_active>
-ALWAYS_INLINE bool DoFieldPut(Thread* self, const ShadowFrame& shadow_frame,
-                              const Instruction* inst, uint16_t inst_data)
+template<FindFieldType find_type, Primitive::Type field_type, bool transaction_active>
+ALWAYS_INLINE bool DoFieldPut(Thread* self,
+                              const ShadowFrame& shadow_frame,
+                              const Instruction* inst,
+                              uint16_t inst_data)
     REQUIRES_SHARED(Locks::mutator_lock_) {
-  const bool do_assignability_check = do_access_check;
+  bool should_report = Runtime::Current()->GetInstrumentation()->HasFieldWriteListeners();
   bool is_static = (find_type == StaticObjectWrite) || (find_type == StaticPrimitiveWrite);
-  uint32_t field_idx = is_static ? inst->VRegB_21c() : inst->VRegC_22c();
-  ArtMethod* method = shadow_frame.GetMethod();
-  ArtField* f = FindFieldFromCode<find_type, do_access_check>(
-      field_idx, method, self, Primitive::ComponentSize(field_type));
-  if (UNLIKELY(f == nullptr)) {
-    CHECK(self->IsExceptionPending());
+  uint32_t vregA = is_static ? inst->VRegA_21c(inst_data) : inst->VRegA_22c(inst_data);
+  bool resolve_field_type = (shadow_frame.GetVRegReference(vregA) != nullptr);
+  ArtField* field = nullptr;
+  MemberOffset offset(0u);
+  bool is_volatile;
+  GetFieldInfo(self,
+               shadow_frame.GetMethod(),
+               reinterpret_cast<const uint16_t*>(inst),
+               is_static,
+               resolve_field_type,
+               &field,
+               &is_volatile,
+               &offset);
+  if (self->IsExceptionPending()) {
     return false;
   }
+
   ObjPtr<mirror::Object> obj;
   if (is_static) {
-    obj = f->GetDeclaringClass();
+    obj = field->GetDeclaringClass();
   } else {
     obj = shadow_frame.GetVRegReference(inst->VRegB_22c(inst_data));
-    if (UNLIKELY(obj == nullptr)) {
-      ThrowNullPointerExceptionForFieldAccess(f, method, false);
-      return false;
+    if (should_report || obj == nullptr) {
+      field = ResolveFieldWithAccessChecks(self,
+                                           Runtime::Current()->GetClassLinker(),
+                                           inst->VRegC_22c(),
+                                           shadow_frame.GetMethod(),
+                                           /* is_static= */ false,
+                                           /* is_put= */ true,
+                                           resolve_field_type);
+      if (UNLIKELY(obj == nullptr)) {
+        ThrowNullPointerExceptionForFieldAccess(
+            field, shadow_frame.GetMethod(), /* is_read= */ false);
+        return false;
+      }
+      // Reload in case suspension happened during field resolution.
+      obj = shadow_frame.GetVRegReference(inst->VRegB_22c(inst_data));
     }
   }
   if (transaction_active && !CheckWriteConstraint(self, obj)) {
     return false;
   }
 
-  uint32_t vregA = is_static ? inst->VRegA_21c(inst_data) : inst->VRegA_22c(inst_data);
   JValue value = GetFieldValue<field_type>(shadow_frame, vregA);
 
   if (transaction_active &&
@@ -522,12 +536,43 @@ ALWAYS_INLINE bool DoFieldPut(Thread* self, const ShadowFrame& shadow_frame,
       !CheckWriteValueConstraint(self, value.GetL())) {
     return false;
   }
+  if (should_report) {
+    return DoFieldPutCommon<field_type, transaction_active>(self,
+                                                            shadow_frame,
+                                                            obj,
+                                                            field,
+                                                            value);
+  }
+#define FIELD_SET(prim, type, jtype) \
+  case Primitive::kPrim ## prim: \
+    if (is_volatile) { \
+      obj->SetField ## type ## Volatile<transaction_active>(offset, value.Get ## jtype()); \
+    } else { \
+      obj->SetField ## type<transaction_active>(offset, value.Get ## jtype()); \
+    } \
+    break;
 
-  return DoFieldPutCommon<field_type, do_assignability_check, transaction_active>(self,
-                                                                                  shadow_frame,
-                                                                                  obj,
-                                                                                  f,
-                                                                                  value);
+  switch (field_type) {
+    FIELD_SET(Boolean, Boolean, Z)
+    FIELD_SET(Byte, Byte, B)
+    FIELD_SET(Char, Char, C)
+    FIELD_SET(Short, Short, S)
+    FIELD_SET(Int, 32, I)
+    FIELD_SET(Long, 64, J)
+    FIELD_SET(Not, Object, L)
+    case Primitive::kPrimVoid: {
+      LOG(FATAL) << "Unreachable " << field_type;
+      break;
+    }
+  }
+#undef FIELD_SET
+
+  if (transaction_active) {
+    if (UNLIKELY(self->IsExceptionPending())) {
+      return false;
+    }
+  }
+  return true;
 }
 
 // Handles string resolution for const-string and const-string-jumbo instructions. Also ensures the
@@ -631,13 +676,16 @@ static inline bool DoLongRemainder(ShadowFrame& shadow_frame,
 
 // Handles filled-new-array and filled-new-array-range instructions.
 // Returns true on success, otherwise throws an exception and returns false.
-template <bool is_range, bool do_access_check, bool transaction_active>
-bool DoFilledNewArray(const Instruction* inst, const ShadowFrame& shadow_frame,
-                      Thread* self, JValue* result);
+template <bool is_range, bool transaction_active>
+bool DoFilledNewArray(const Instruction* inst,
+                      const ShadowFrame& shadow_frame,
+                      Thread* self,
+                      JValue* result);
 
 // Handles packed-switch instruction.
 // Returns the branch offset to the next instruction to execute.
-static inline int32_t DoPackedSwitch(const Instruction* inst, const ShadowFrame& shadow_frame,
+static inline int32_t DoPackedSwitch(const Instruction* inst,
+                                     const ShadowFrame& shadow_frame,
                                      uint16_t inst_data)
     REQUIRES_SHARED(Locks::mutator_lock_) {
   DCHECK(inst->Opcode() == Instruction::PACKED_SWITCH);
@@ -755,34 +803,6 @@ void ArtInterpreterToCompiledCodeBridge(Thread* self,
                                         uint16_t arg_offset,
                                         JValue* result);
 
-static inline bool IsStringInit(const DexFile* dex_file, uint32_t method_idx)
-    REQUIRES_SHARED(Locks::mutator_lock_) {
-  const dex::MethodId& method_id = dex_file->GetMethodId(method_idx);
-  const char* class_name = dex_file->StringByTypeIdx(method_id.class_idx_);
-  const char* method_name = dex_file->GetMethodName(method_id);
-  // Instead of calling ResolveMethod() which has suspend point and can trigger
-  // GC, look up the method symbolically.
-  // Compare method's class name and method name against string init.
-  // It's ok since it's not allowed to create your own java/lang/String.
-  // TODO: verify that assumption.
-  if ((strcmp(class_name, "Ljava/lang/String;") == 0) &&
-      (strcmp(method_name, "<init>") == 0)) {
-    return true;
-  }
-  return false;
-}
-
-static inline bool IsStringInit(const Instruction* instr, ArtMethod* caller)
-    REQUIRES_SHARED(Locks::mutator_lock_) {
-  if (instr->Opcode() == Instruction::INVOKE_DIRECT ||
-      instr->Opcode() == Instruction::INVOKE_DIRECT_RANGE) {
-    uint16_t callee_method_idx = (instr->Opcode() == Instruction::INVOKE_DIRECT_RANGE) ?
-        instr->VRegB_3rc() : instr->VRegB_35c();
-    return IsStringInit(caller->GetDexFile(), callee_method_idx);
-  }
-  return false;
-}
-
 // Set string value created from StringFactory.newStringFromXXX() into all aliases of
 // StringFactory.newEmptyString().
 void SetStringInitValueToAllAliases(ShadowFrame* shadow_frame,
