@@ -23,7 +23,6 @@
 #include <unistd.h>
 
 #include "android-base/strings.h"
-
 #include "art_method-inl.h"
 #include "base/compiler_filter.h"
 #include "base/enums.h"
@@ -32,6 +31,7 @@
 #include "base/stl_util.h"
 #include "base/systrace.h"
 #include "base/time_utils.h"
+#include "base/unix_file/fd_file.h"
 #include "class_table-inl.h"
 #include "dex/dex_file_loader.h"
 #include "dex_reference_collection.h"
@@ -136,21 +136,20 @@ void ProfileSaver::Run() {
   {
     MutexLock mu(self, wait_lock_);
 
-    const uint64_t end_time = NanoTime() + MsToNs(force_early_first_save
+    const uint64_t sleep_time = MsToNs(force_early_first_save
       ? options_.GetMinFirstSaveMs()
       : options_.GetSaveResolvedClassesDelayMs());
-    while (!Runtime::Current()->GetStartupCompleted()) {
+    const uint64_t start_time = NanoTime();
+    const uint64_t end_time = start_time + sleep_time;
+    while (!Runtime::Current()->GetStartupCompleted() || force_early_first_save) {
       const uint64_t current_time = NanoTime();
       if (current_time >= end_time) {
         break;
       }
       period_condition_.TimedWait(self, NsToMs(end_time - current_time), 0);
     }
-    total_ms_of_sleep_ += options_.GetSaveResolvedClassesDelayMs();
+    total_ms_of_sleep_ += NsToMs(NanoTime() - start_time);
   }
-  // Tell the runtime that startup is completed if it has not already been notified.
-  // TODO: We should use another thread to do this in case the profile saver is not running.
-  Runtime::Current()->NotifyStartupCompleted();
 
   FetchAndCacheResolvedClassesAndMethods(/*startup=*/ true);
 
@@ -869,11 +868,17 @@ bool ProfileSaver::ProcessProfilingInfo(
     }
     {
       ProfileCompilationInfo info(Runtime::Current()->GetArenaPool(),
-                                  /*for_boot_image=*/ options_.GetProfileBootClassPath());
-      if (!info.Load(filename, /*clear_if_invalid=*/ true)) {
+                                  /*for_boot_image=*/options_.GetProfileBootClassPath());
+      // Load the existing profile before saving.
+      // If the file is updated between `Load` and `Save`, the update will be lost. This is
+      // acceptable. The main reason is that the lost entries will eventually come back if the user
+      // keeps using the same methods, or they won't be needed if the user doesn't use the same
+      // methods again.
+      if (!info.Load(filename, /*clear_if_invalid=*/true)) {
         LOG(WARNING) << "Could not forcefully load profile " << filename;
         continue;
       }
+
       uint64_t last_save_number_of_methods = info.GetNumberOfMethods();
       uint64_t last_save_number_of_classes = info.GetNumberOfResolvedClasses();
       VLOG(profiler) << "last_save_number_of_methods=" << last_save_number_of_methods
