@@ -23,7 +23,7 @@
 #include "base/macros.h"
 #include "utils/arm/managed_register_arm.h"
 
-namespace art {
+namespace art HIDDEN {
 namespace arm {
 
 //
@@ -199,6 +199,10 @@ ManagedRegister ArmManagedRuntimeCallingConvention::MethodRegister() {
   return ArmManagedRegister::FromCoreRegister(R0);
 }
 
+ManagedRegister ArmManagedRuntimeCallingConvention::ArgumentRegisterForMethodExitHook() {
+  return ArmManagedRegister::FromCoreRegister(R2);
+}
+
 void ArmManagedRuntimeCallingConvention::ResetIterator(FrameOffset displacement) {
   ManagedRuntimeCallingConvention::ResetIterator(displacement);
   gpr_index_ = 1u;  // Skip r0 for ArtMethod*
