@@ -45,6 +45,7 @@
 #include "intern_table.h"
 #include "lock_word.h"
 #include "mirror/dex_cache.h"
+#include "oat.h"
 #include "oat_file.h"
 #include "obj_ptr.h"
 
@@ -229,18 +230,6 @@ class ImageWriter final {
   };
   friend std::ostream& operator<<(std::ostream& stream, NativeObjectRelocationType type);
 
-  enum class StubType {
-    kJNIDlsymLookupTrampoline,
-    kJNIDlsymLookupCriticalTrampoline,
-    kQuickGenericJNITrampoline,
-    kQuickIMTConflictTrampoline,
-    kQuickResolutionTrampoline,
-    kQuickToInterpreterBridge,
-    kNterpTrampoline,
-    kLast = kNterpTrampoline,
-  };
-  friend std::ostream& operator<<(std::ostream& stream, StubType stub_type);
-
   static constexpr size_t kBinBits =
       MinimumBitsToStore<uint32_t>(static_cast<size_t>(Bin::kMirrorCount) - 1);
   // uint32 = typeof(lockword_)
@@ -462,6 +451,25 @@ class ImageWriter final {
       REQUIRES_SHARED(Locks::mutator_lock_);
   void CalculateObjectBinSlots(mirror::Object* obj)
       REQUIRES_SHARED(Locks::mutator_lock_);
+  // Undo the changes of CalculateNewObjectOffsets.
+  void ResetObjectOffsets() REQUIRES_SHARED(Locks::mutator_lock_);
+  // Reset and calculate new offsets with dirty objects optimization.
+  // Does nothing if dirty object offsets don't match with current offsets.
+  void TryRecalculateOffsetsWithDirtyObjects() REQUIRES_SHARED(Locks::mutator_lock_);
+
+  // Dirty object data from dirty-image-objects.
+  struct DirtyEntry {
+    uint32_t descriptor_hash = 0;
+    bool is_class = false;
+    uint32_t sort_key = 0;
+  };
+  // Parse dirty-image-objects into (offset->entry) map. Returns nullopt on parse error.
+  static std::optional<HashMap<uint32_t, DirtyEntry>> ParseDirtyObjectOffsets(
+      const HashSet<std::string>& dirty_image_objects) REQUIRES_SHARED(Locks::mutator_lock_);
+  // Get all objects that match dirty_entries by offset. Returns nullopt if there is a mismatch.
+  // Map values are sort_keys from DirtyEntry.
+  std::optional<HashMap<mirror::Object*, uint32_t>> MatchDirtyObjectOffsets(
+      const HashMap<uint32_t, DirtyEntry>& dirty_entries) REQUIRES_SHARED(Locks::mutator_lock_);
 
   // Creates the contiguous image in memory and adjusts pointers.
   void CopyAndFixupNativeData(size_t oat_index) REQUIRES_SHARED(Locks::mutator_lock_);
@@ -685,9 +693,14 @@ class ImageWriter final {
   // Map of dex files to the indexes of oat files that they were compiled into.
   const HashMap<const DexFile*, size_t>& dex_file_oat_index_map_;
 
-  // Set of objects known to be dirty in the image. Can be nullptr if there are none.
+  // Set of classes/objects known to be dirty in the image. Can be nullptr if there are none.
+  // For old dirty-image-objects format this set contains descriptors of dirty classes.
+  // For new format -- a set of dirty object offsets and descriptor hashes.
   const HashSet<std::string>* dirty_image_objects_;
 
+  // Dirty object instances and their sort keys parsed from dirty_image_object_
+  HashMap<mirror::Object*, uint32_t> dirty_objects_;
+
   // Objects are guaranteed to not cross the region size boundary.
   size_t region_size_ = 0u;
 
@@ -697,7 +710,6 @@ class ImageWriter final {
   class FixupClassVisitor;
   class FixupRootVisitor;
   class FixupVisitor;
-  class ImageFileGuard;
   class LayoutHelper;
   class NativeLocationVisitor;
   class PruneClassesVisitor;
@@ -712,7 +724,6 @@ class ImageWriter final {
 
 std::ostream& operator<<(std::ostream& stream, ImageWriter::Bin bin);
 std::ostream& operator<<(std::ostream& stream, ImageWriter::NativeObjectRelocationType type);
-std::ostream& operator<<(std::ostream& stream, ImageWriter::StubType stub_type);
 
 }  // namespace linker
 }  // namespace art
