@@ -27,6 +27,7 @@
 #include "art_method-inl.h"
 #include "base/enums.h"
 #include "base/logging.h"  // For VLOG.
+#include "base/sdk_version.h"
 #include "base/utils.h"
 #include "class-inl.h"
 #include "class_ext-inl.h"
@@ -59,10 +60,6 @@
 
 namespace art {
 
-// TODO: move to own CC file?
-constexpr size_t BitString::kBitSizeAtPosition[BitString::kCapacity];
-constexpr size_t BitString::kCapacity;
-
 namespace mirror {
 
 using android::base::StringPrintf;
@@ -1436,7 +1433,7 @@ ArtField* Class::FindStaticField(ObjPtr<mirror::DexCache> dex_cache, uint32_t fi
 void Class::ClearSkipAccessChecksFlagOnAllMethods(PointerSize pointer_size) {
   DCHECK(IsVerified());
   for (auto& m : GetMethods(pointer_size)) {
-    if (!m.IsNative() && m.IsInvokable()) {
+    if (m.IsManagedAndInvokable()) {
       m.ClearSkipAccessChecks();
     }
   }
@@ -1445,7 +1442,7 @@ void Class::ClearSkipAccessChecksFlagOnAllMethods(PointerSize pointer_size) {
 void Class::ClearMustCountLocksFlagOnAllMethods(PointerSize pointer_size) {
   DCHECK(IsVerified());
   for (auto& m : GetMethods(pointer_size)) {
-    if (!m.IsNative() && m.IsInvokable()) {
+    if (m.IsManagedAndInvokable()) {
       m.ClearMustCountLocks();
     }
   }
@@ -1454,7 +1451,7 @@ void Class::ClearMustCountLocksFlagOnAllMethods(PointerSize pointer_size) {
 void Class::ClearDontCompileFlagOnAllMethods(PointerSize pointer_size) {
   DCHECK(IsVerified());
   for (auto& m : GetMethods(pointer_size)) {
-    if (!m.IsNative() && m.IsInvokable()) {
+    if (m.IsManagedAndInvokable()) {
       m.ClearDontCompile();
     }
   }
@@ -1463,7 +1460,7 @@ void Class::ClearDontCompileFlagOnAllMethods(PointerSize pointer_size) {
 void Class::SetSkipAccessChecksFlagOnAllMethods(PointerSize pointer_size) {
   DCHECK(IsVerified());
   for (auto& m : GetMethods(pointer_size)) {
-    if (!m.IsNative() && m.IsInvokable()) {
+    if (m.IsManagedAndInvokable()) {
       m.SetSkipAccessChecks();
     }
   }
@@ -2132,6 +2129,19 @@ size_t Class::GetMethodIdOffset(ArtMethod* method, PointerSize pointer_size) {
   return res;
 }
 
+bool Class::CheckIsVisibleWithTargetSdk(Thread* self) {
+  uint32_t targetSdkVersion = Runtime::Current()->GetTargetSdkVersion();
+  if (IsSdkVersionSetAndAtMost(targetSdkVersion, SdkVersion::kT)) {
+    ObjPtr<mirror::Class> java_lang_ClassValue =
+        WellKnownClasses::ToClass(WellKnownClasses::java_lang_ClassValue);
+    if (this == java_lang_ClassValue.Ptr()) {
+      self->ThrowNewException("Ljava/lang/ClassNotFoundException;", "java.lang.ClassValue");
+      return false;
+    }
+  }
+  return true;
+}
+
 ArtMethod* Class::FindAccessibleInterfaceMethod(ArtMethod* implementation_method,
                                                 PointerSize pointer_size)
     REQUIRES_SHARED(Locks::mutator_lock_) {
