@@ -340,6 +340,26 @@ ALWAYS_INLINE inline static void SetFieldValue(ObjPtr<mirror::Object> o,
   }
 }
 
+ALWAYS_INLINE inline static bool ThrowIAEIfRecordFinalField(ObjPtr<mirror::Field> field)
+    REQUIRES_SHARED(Locks::mutator_lock_) {
+  if (!(field->IsFinal())) {
+    return false;
+  }
+  ObjPtr<mirror::Class> declaring_class = field->GetDeclaringClass();
+  DCHECK(declaring_class != nullptr);
+  if (!(declaring_class->IsRecordClass())) {
+    return false;
+  }
+
+  ThrowIllegalAccessException(
+          StringPrintf("Cannot set %s field %s of record class %s",
+              PrettyJavaAccessFlags(field->GetAccessFlags()).c_str(),
+              ArtField::PrettyField(field->GetArtField()).c_str(),
+              declaring_class->PrettyClass().c_str()).c_str());
+
+  return true;
+}
+
 static void Field_set(JNIEnv* env, jobject javaField, jobject javaObj, jobject javaValue) {
   ScopedFastNativeObjectAccess soa(env);
   ObjPtr<mirror::Field> f = soa.Decode<mirror::Field>(javaField);
@@ -349,6 +369,10 @@ static void Field_set(JNIEnv* env, jobject javaField, jobject javaObj, jobject j
     DCHECK(soa.Self()->IsExceptionPending());
     return;
   }
+  if (ThrowIAEIfRecordFinalField(f)) {
+    DCHECK(soa.Self()->IsExceptionPending());
+    return;
+  }
   ObjPtr<mirror::Class> field_type;
   const char* field_type_descriptor = f->GetArtField()->GetTypeDescriptor();
   Primitive::Type field_prim_type = Primitive::GetType(field_type_descriptor[0]);
@@ -389,6 +413,10 @@ static void SetPrimitiveField(JNIEnv* env,
   if (!CheckReceiver(soa, javaObj, &f, &o)) {
     return;
   }
+  if (ThrowIAEIfRecordFinalField(f)) {
+    DCHECK(soa.Self()->IsExceptionPending());
+    return;
+  }
   Primitive::Type field_type = f->GetTypeAsPrimitiveType();
   if (UNLIKELY(field_type == Primitive::kPrimNot)) {
     ThrowIllegalArgumentException(
@@ -491,7 +519,7 @@ static jobjectArray Field_getDeclaredAnnotations(JNIEnv* env, jobject javaField)
   if (field->GetDeclaringClass()->IsProxyClass()) {
     // Return an empty array instead of a null pointer.
     ObjPtr<mirror::Class> annotation_array_class =
-        soa.Decode<mirror::Class>(WellKnownClasses::java_lang_annotation_Annotation__array);
+        WellKnownClasses::ToClass(WellKnownClasses::java_lang_annotation_Annotation__array);
     ObjPtr<mirror::ObjectArray<mirror::Object>> empty_array =
         mirror::ObjectArray<mirror::Object>::Alloc(soa.Self(), annotation_array_class, 0);
     return soa.AddLocalReference<jobjectArray>(empty_array);
