@@ -19,6 +19,7 @@
 
 #include "arch/x86/instruction_set_features_x86.h"
 #include "base/enums.h"
+#include "base/macros.h"
 #include "code_generator.h"
 #include "dex/dex_file_types.h"
 #include "driver/compiler_options.h"
@@ -26,7 +27,7 @@
 #include "parallel_move_resolver.h"
 #include "utils/x86/assembler_x86.h"
 
-namespace art {
+namespace art HIDDEN {
 namespace x86 {
 
 // Use a local definition to prevent copying mistakes.
@@ -47,6 +48,61 @@ static constexpr XmmRegister kRuntimeParameterFpuRegisters[] = { XMM0, XMM1, XMM
 static constexpr size_t kRuntimeParameterFpuRegistersLength =
     arraysize(kRuntimeParameterFpuRegisters);
 
+#define UNIMPLEMENTED_INTRINSIC_LIST_X86(V) \
+  V(MathRoundDouble)                        \
+  V(FloatIsInfinite)                        \
+  V(DoubleIsInfinite)                       \
+  V(IntegerHighestOneBit)                   \
+  V(LongHighestOneBit)                      \
+  V(LongDivideUnsigned)                     \
+  V(CRC32Update)                            \
+  V(CRC32UpdateBytes)                       \
+  V(CRC32UpdateByteBuffer)                  \
+  V(FP16ToFloat)                            \
+  V(FP16ToHalf)                             \
+  V(FP16Floor)                              \
+  V(FP16Ceil)                               \
+  V(FP16Rint)                               \
+  V(FP16Greater)                            \
+  V(FP16GreaterEquals)                      \
+  V(FP16Less)                               \
+  V(FP16LessEquals)                         \
+  V(FP16Compare)                            \
+  V(FP16Min)                                \
+  V(FP16Max)                                \
+  V(MathMultiplyHigh)                       \
+  V(StringStringIndexOf)                    \
+  V(StringStringIndexOfAfter)               \
+  V(StringBufferAppend)                     \
+  V(StringBufferLength)                     \
+  V(StringBufferToString)                   \
+  V(StringBuilderAppendObject)              \
+  V(StringBuilderAppendString)              \
+  V(StringBuilderAppendCharSequence)        \
+  V(StringBuilderAppendCharArray)           \
+  V(StringBuilderAppendBoolean)             \
+  V(StringBuilderAppendChar)                \
+  V(StringBuilderAppendInt)                 \
+  V(StringBuilderAppendLong)                \
+  V(StringBuilderAppendFloat)               \
+  V(StringBuilderAppendDouble)              \
+  V(StringBuilderLength)                    \
+  V(StringBuilderToString)                  \
+  /* 1.8 */                                 \
+  V(UnsafeGetAndAddInt)                     \
+  V(UnsafeGetAndAddLong)                    \
+  V(UnsafeGetAndSetInt)                     \
+  V(UnsafeGetAndSetLong)                    \
+  V(UnsafeGetAndSetObject)                  \
+  V(MethodHandleInvokeExact)                \
+  V(MethodHandleInvoke)                     \
+  /* OpenJDK 11 */                          \
+  V(JdkUnsafeGetAndAddInt)                  \
+  V(JdkUnsafeGetAndAddLong)                 \
+  V(JdkUnsafeGetAndSetInt)                  \
+  V(JdkUnsafeGetAndSetLong)                 \
+  V(JdkUnsafeGetAndSetObject)
+
 class InvokeRuntimeCallingConvention : public CallingConvention<Register, XmmRegister> {
  public:
   InvokeRuntimeCallingConvention()
@@ -196,7 +252,9 @@ class LocationsBuilderX86 : public HGraphVisitor {
   void HandleInvoke(HInvoke* invoke);
   void HandleCondition(HCondition* condition);
   void HandleShift(HBinaryOperation* instruction);
-  void HandleFieldSet(HInstruction* instruction, const FieldInfo& field_info);
+  void HandleFieldSet(HInstruction* instruction,
+                      const FieldInfo& field_info,
+                      WriteBarrierKind write_barrier_kind);
   void HandleFieldGet(HInstruction* instruction, const FieldInfo& field_info);
   bool CpuHasAvxFeatureFlag();
   bool CpuHasAvx2FeatureFlag();
@@ -249,7 +307,8 @@ class InstructionCodeGeneratorX86 : public InstructionCodeGenerator {
                       Address field_addr,
                       Register base,
                       bool is_volatile,
-                      bool value_can_be_null);
+                      bool value_can_be_null,
+                      WriteBarrierKind write_barrier_kind);
 
  private:
   // Generate code for the given suspend check. If not null, `successor`
@@ -279,7 +338,8 @@ class InstructionCodeGeneratorX86 : public InstructionCodeGenerator {
 
   void HandleFieldSet(HInstruction* instruction,
                       const FieldInfo& field_info,
-                      bool value_can_be_null);
+                      bool value_can_be_null,
+                      WriteBarrierKind write_barrier_kind);
   void HandleFieldGet(HInstruction* instruction, const FieldInfo& field_info);
 
   // Generate a heap reference load using one register `out`:
@@ -519,11 +579,8 @@ class CodeGeneratorX86 : public CodeGenerator {
   void EmitJitRootPatches(uint8_t* code, const uint8_t* roots_data) override;
 
   // Emit a write barrier.
-  void MarkGCCard(Register temp,
-                  Register card,
-                  Register object,
-                  Register value,
-                  bool value_can_be_null);
+  void MarkGCCard(
+      Register temp, Register card, Register object, Register value, bool emit_null_check);
 
   void GenerateMemoryBarrier(MemBarrierKind kind);
 
