@@ -19,30 +19,55 @@
 #include "base/arena_bit_vector.h"
 #include "base/array_ref.h"
 #include "base/bit_vector-inl.h"
+#include "base/globals.h"
 #include "base/logging.h"
 #include "base/scoped_arena_allocator.h"
 #include "base/scoped_arena_containers.h"
 #include "common_dominator.h"
 #include "nodes.h"
 
-namespace art {
+namespace art HIDDEN {
 
 bool CodeSinking::Run() {
-  HBasicBlock* exit = graph_->GetExitBlock();
-  if (exit == nullptr) {
+  if (graph_->GetExitBlock() == nullptr) {
     // Infinite loop, just bail.
     return false;
   }
+
+  UncommonBranchSinking();
+  ReturnSinking();
+  return true;
+}
+
+void CodeSinking::UncommonBranchSinking() {
+  HBasicBlock* exit = graph_->GetExitBlock();
+  DCHECK(exit != nullptr);
   // TODO(ngeoffray): we do not profile branches yet, so use throw instructions
   // as an indicator of an uncommon branch.
   for (HBasicBlock* exit_predecessor : exit->GetPredecessors()) {
     HInstruction* last = exit_predecessor->GetLastInstruction();
+
+    // TryBoundary instructions are sometimes inserted between the last instruction (e.g. Throw,
+    // Return) and Exit. We don't want to use that instruction for our "uncommon branch" heuristic
+    // because they are not as good an indicator as throwing branches, so we skip them and fetch the
+    // actual last instruction.
+    if (last->IsTryBoundary()) {
+      // We have an exit try boundary. Fetch the previous instruction.
+      DCHECK(!last->AsTryBoundary()->IsEntry());
+      if (last->GetPrevious() == nullptr) {
+        DCHECK(exit_predecessor->IsSingleTryBoundary());
+        exit_predecessor = exit_predecessor->GetSinglePredecessor();
+        last = exit_predecessor->GetLastInstruction();
+      } else {
+        last = last->GetPrevious();
+      }
+    }
+
     // Any predecessor of the exit that does not return, throws an exception.
     if (!last->IsReturn() && !last->IsReturnVoid()) {
       SinkCodeToUncommonBranch(exit_predecessor);
     }
   }
-  return true;
 }
 
 static bool IsInterestingInstruction(HInstruction* instruction) {
@@ -88,7 +113,7 @@ static bool IsInterestingInstruction(HInstruction* instruction) {
 
   // We can only store on local allocations. Other heap references can
   // be escaping. Note that allocations can escape too, but we only move
-  // allocations if their users can move to, or are in the list of
+  // allocations if their users can move too, or are in the list of
   // post dominated blocks.
   if (instruction->IsInstanceFieldSet()) {
     if (!instruction->InputAt(0)->IsNewInstance()) {
@@ -102,7 +127,7 @@ static bool IsInterestingInstruction(HInstruction* instruction) {
     }
   }
 
-  // Heap accesses cannot go pass instructions that have memory side effects, which
+  // Heap accesses cannot go past instructions that have memory side effects, which
   // we are not tracking here. Note that the load/store elimination optimization
   // runs before this optimization, and should have removed interesting ones.
   // In theory, we could handle loads of local allocations, but this is currently
@@ -171,7 +196,6 @@ static bool ShouldFilterUse(HInstruction* instruction,
   return false;
 }
 
-
 // Find the ideal position for moving `instruction`. If `filter` is true,
 // we filter out store instructions to that instruction, which are processed
 // first in the step (3) of the sinking algorithm.
@@ -210,56 +234,52 @@ static HInstruction* FindIdealPosition(HInstruction* instruction,
     return nullptr;
   }
 
-  // Move to the first dominator not in a loop, if we can.
-  while (target_block->IsInLoop()) {
+  // Move to the first dominator not in a loop, if we can. We only do this if we are trying to hoist
+  // `instruction` out of a loop it wasn't a part of.
+  const HLoopInformation* loop_info = instruction->GetBlock()->GetLoopInformation();
+  while (target_block->IsInLoop() && target_block->GetLoopInformation() != loop_info) {
     if (!post_dominated.IsBitSet(target_block->GetDominator()->GetBlockId())) {
       break;
     }
     target_block = target_block->GetDominator();
     DCHECK(target_block != nullptr);
   }
-  const bool was_in_loop = target_block->IsInLoop();
 
-  // For throwing instructions we can move them into:
-  //   * Blocks that are not part of a try
-  //     * Catch blocks are suitable as well, as long as they are not part of an outer try.
-  //   * Blocks that are part of the same try that the instrucion was already in.
-  //
-  // We cannot move an instruction that can throw into a try that said instruction is not a part of
-  // already, as that would mean it will throw into a different catch block. If we detect that
-  // `target_block` is not a valid block to move `instruction` to, we traverse up the dominator tree
-  // to find if we have a suitable block.
-  while (instruction->CanThrow() && target_block->GetTryCatchInformation() != nullptr) {
-    if (target_block->IsCatchBlock()) {
-      // If the catch block has an xhandler, it means it is inside of an outer try.
-      const bool inside_of_another_try_catch = target_block->GetSuccessors().size() != 1;
-      if (!inside_of_another_try_catch) {
-        // If we have a catch block, it's okay to sink as long as that catch is not inside of
-        // another try catch.
-        break;
+  if (instruction->CanThrow()) {
+    // Consistency check: We shouldn't land in a loop if we weren't in one before traversing up the
+    // dominator tree regarding try catches.
+    const bool was_in_loop = target_block->IsInLoop();
+
+    // We cannot move an instruction that can throw into a try that said instruction is not a part
+    // of already, as that would mean it will throw into a different catch block. In short, for
+    // throwing instructions:
+    // * If the throwing instruction is part of a try, they should only be sunk into that same try.
+    // * If the throwing instruction is not part of any try, they shouldn't be sunk to any try.
+    if (instruction->GetBlock()->IsTryBlock()) {
+      const HTryBoundary& try_entry =
+          instruction->GetBlock()->GetTryCatchInformation()->GetTryEntry();
+      while (!(target_block->IsTryBlock() &&
+               try_entry.HasSameExceptionHandlersAs(
+                   target_block->GetTryCatchInformation()->GetTryEntry()))) {
+        target_block = target_block->GetDominator();
+        if (!post_dominated.IsBitSet(target_block->GetBlockId())) {
+          // We couldn't find a suitable block.
+          return nullptr;
+        }
       }
     } else {
-      DCHECK(target_block->IsTryBlock());
-      if (instruction->GetBlock()->IsTryBlock() &&
-          instruction->GetBlock()->GetTryCatchInformation()->GetTryEntry().GetId() ==
-              target_block->GetTryCatchInformation()->GetTryEntry().GetId()) {
-        // Sink within the same try block is allowed.
-        break;
+      // Search for the first block also not in a try block
+      while (target_block->IsTryBlock()) {
+        target_block = target_block->GetDominator();
+        if (!post_dominated.IsBitSet(target_block->GetBlockId())) {
+          // We couldn't find a suitable block.
+          return nullptr;
+        }
       }
     }
-    // We are now in the case where we would be moving to a different try. Since we don't want
-    // that, traverse up the dominator tree to find a suitable block.
-    if (!post_dominated.IsBitSet(target_block->GetDominator()->GetBlockId())) {
-      // We couldn't find a suitable block.
-      return nullptr;
-    }
-    target_block = target_block->GetDominator();
-    DCHECK(target_block != nullptr);
-  }
 
-  // We shouldn't land in a loop if we weren't in one before traversing up the dominator tree
-  // regarding try catches.
-  DCHECK_IMPLIES(target_block->IsInLoop(), was_in_loop);
+    DCHECK_IMPLIES(target_block->IsInLoop(), was_in_loop);
+  }
 
   // Find insertion position. No need to filter anymore, as we have found a
   // target block.
@@ -271,10 +291,21 @@ static HInstruction* FindIdealPosition(HInstruction* instruction,
     }
   }
   for (const HUseListNode<HEnvironment*>& use : instruction->GetEnvUses()) {
-    HInstruction* user = use.GetUser()->GetHolder();
+    HEnvironment* env = use.GetUser();
+    HInstruction* user = env->GetHolder();
     if (user->GetBlock() == target_block &&
         (insert_pos == nullptr || user->StrictlyDominates(insert_pos))) {
-      insert_pos = user;
+      if (target_block->IsCatchBlock() && target_block->GetFirstInstruction() == user) {
+        // We can sink the instructions past the environment setting Nop. If we do that, we have to
+        // remove said instruction from the environment. Since we know that we will be sinking the
+        // instruction to this block and there are no more instructions to consider, we can safely
+        // remove it from the environment now.
+        DCHECK(target_block->GetFirstInstruction()->IsNop());
+        env->RemoveAsUserOfInput(use.GetIndex());
+        env->SetRawEnvAt(use.GetIndex(), /*instruction=*/ nullptr);
+      } else {
+        insert_pos = user;
+      }
     }
   }
   if (insert_pos == nullptr) {
@@ -310,8 +341,8 @@ void CodeSinking::SinkCodeToUncommonBranch(HBasicBlock* end_block) {
   ScopedArenaVector<HInstruction*> move_in_order(allocator.Adapter(kArenaAllocMisc));
 
   // Step (1): Visit post order to get a subset of blocks post dominated by `end_block`.
-  // TODO(ngeoffray): Getting the full set of post-dominated shoud be done by
-  // computint the post dominator tree, but that could be too time consuming. Also,
+  // TODO(ngeoffray): Getting the full set of post-dominated should be done by
+  // computing the post dominator tree, but that could be too time consuming. Also,
   // we should start the analysis from blocks dominated by an uncommon branch, but we
   // don't profile branches yet.
   bool found_block = false;
@@ -321,45 +352,43 @@ void CodeSinking::SinkCodeToUncommonBranch(HBasicBlock* end_block) {
       post_dominated.SetBit(block->GetBlockId());
     } else if (found_block) {
       bool is_post_dominated = true;
-      if (block->GetSuccessors().empty()) {
-        // We currently bail for loops.
-        is_post_dominated = false;
-      } else {
-        // BasicBlock that are try entries look like this:
-        //   BasicBlock i:
-        //     instr 1
-        //     ...
-        //     instr N
-        //     TryBoundary kind:entry ---Try begins here---
-        //
-        // Due to how our BasicBlocks are structured, BasicBlock i will have an xhandler successor
-        // since we are starting a try. If we use `GetSuccessors` for this case, we will check if
-        // the catch block is post_dominated.
-        //
-        // However, this catch block doesn't matter: when we sink the instruction into that
-        // BasicBlock i, we do it before the TryBoundary (i.e. outside of the try and outside the
-        // catch's domain). We can ignore catch blocks using `GetNormalSuccessors` to sink code
-        // right before the start of a try block.
-        //
-        // On the other side of the coin, BasicBlock that are try exits look like this:
-        //   BasicBlock j:
-        //     instr 1
-        //     ...
-        //     instr N
-        //     TryBoundary kind:exit ---Try ends here---
-        //
-        // If we sink to these basic blocks we would be sinking inside of the try so we would like
-        // to check the catch block for post dominance.
-        const bool ends_with_try_boundary_entry =
-            block->EndsWithTryBoundary() && block->GetLastInstruction()->AsTryBoundary()->IsEntry();
-        ArrayRef<HBasicBlock* const> successors =
-            ends_with_try_boundary_entry ? block->GetNormalSuccessors() :
-                                           ArrayRef<HBasicBlock* const>(block->GetSuccessors());
-        for (HBasicBlock* successor : successors) {
-          if (!post_dominated.IsBitSet(successor->GetBlockId())) {
-            is_post_dominated = false;
-            break;
-          }
+      DCHECK_NE(block, graph_->GetExitBlock())
+          << "We shouldn't encounter the exit block after `end_block`.";
+
+      // BasicBlock that are try entries look like this:
+      //   BasicBlock i:
+      //     instr 1
+      //     ...
+      //     instr N
+      //     TryBoundary kind:entry ---Try begins here---
+      //
+      // Due to how our BasicBlocks are structured, BasicBlock i will have an xhandler successor
+      // since we are starting a try. If we use `GetSuccessors` for this case, we will check if
+      // the catch block is post_dominated.
+      //
+      // However, this catch block doesn't matter: when we sink the instruction into that
+      // BasicBlock i, we do it before the TryBoundary (i.e. outside of the try and outside the
+      // catch's domain). We can ignore catch blocks using `GetNormalSuccessors` to sink code
+      // right before the start of a try block.
+      //
+      // On the other side of the coin, BasicBlock that are try exits look like this:
+      //   BasicBlock j:
+      //     instr 1
+      //     ...
+      //     instr N
+      //     TryBoundary kind:exit ---Try ends here---
+      //
+      // If we sink to these basic blocks we would be sinking inside of the try so we would like
+      // to check the catch block for post dominance.
+      const bool ends_with_try_boundary_entry =
+          block->EndsWithTryBoundary() && block->GetLastInstruction()->AsTryBoundary()->IsEntry();
+      ArrayRef<HBasicBlock* const> successors =
+          ends_with_try_boundary_entry ? block->GetNormalSuccessors() :
+                                         ArrayRef<HBasicBlock* const>(block->GetSuccessors());
+      for (HBasicBlock* successor : successors) {
+        if (!post_dominated.IsBitSet(successor->GetBlockId())) {
+          is_post_dominated = false;
+          break;
         }
       }
       if (is_post_dominated) {
@@ -509,4 +538,79 @@ void CodeSinking::SinkCodeToUncommonBranch(HBasicBlock* end_block) {
   }
 }
 
+void CodeSinking::ReturnSinking() {
+  HBasicBlock* exit = graph_->GetExitBlock();
+  DCHECK(exit != nullptr);
+
+  int number_of_returns = 0;
+  bool saw_return = false;
+  for (HBasicBlock* pred : exit->GetPredecessors()) {
+    // TODO(solanes): We might have Return/ReturnVoid->TryBoundary->Exit. We can theoretically
+    // handle them and move them out of the TryBoundary. However, it is a border case and it adds
+    // codebase complexity.
+    if (pred->GetLastInstruction()->IsReturn() || pred->GetLastInstruction()->IsReturnVoid()) {
+      saw_return |= pred->GetLastInstruction()->IsReturn();
+      ++number_of_returns;
+    }
+  }
+
+  if (number_of_returns < 2) {
+    // Nothing to do.
+    return;
+  }
+
+  // `new_block` will coalesce the Return instructions into Phi+Return, or the ReturnVoid
+  // instructions into a ReturnVoid.
+  HBasicBlock* new_block = new (graph_->GetAllocator()) HBasicBlock(graph_, exit->GetDexPc());
+  if (saw_return) {
+    HPhi* new_phi = nullptr;
+    for (size_t i = 0; i < exit->GetPredecessors().size(); /*++i in loop*/) {
+      HBasicBlock* pred = exit->GetPredecessors()[i];
+      if (!pred->GetLastInstruction()->IsReturn()) {
+        ++i;
+        continue;
+      }
+
+      HReturn* ret = pred->GetLastInstruction()->AsReturn();
+      if (new_phi == nullptr) {
+        // Create the new_phi, if we haven't done so yet. We do it here since we need to know the
+        // type to assign to it.
+        new_phi = new (graph_->GetAllocator()) HPhi(graph_->GetAllocator(),
+                                                    kNoRegNumber,
+                                                    /*number_of_inputs=*/0,
+                                                    ret->InputAt(0)->GetType());
+        new_block->AddPhi(new_phi);
+      }
+      new_phi->AddInput(ret->InputAt(0));
+      pred->ReplaceAndRemoveInstructionWith(ret,
+                                            new (graph_->GetAllocator()) HGoto(ret->GetDexPc()));
+      pred->ReplaceSuccessor(exit, new_block);
+      // Since we are removing a predecessor, there's no need to increment `i`.
+    }
+    new_block->AddInstruction(new (graph_->GetAllocator()) HReturn(new_phi, exit->GetDexPc()));
+  } else {
+    for (size_t i = 0; i < exit->GetPredecessors().size(); /*++i in loop*/) {
+      HBasicBlock* pred = exit->GetPredecessors()[i];
+      if (!pred->GetLastInstruction()->IsReturnVoid()) {
+        ++i;
+        continue;
+      }
+
+      HReturnVoid* ret = pred->GetLastInstruction()->AsReturnVoid();
+      pred->ReplaceAndRemoveInstructionWith(ret,
+                                            new (graph_->GetAllocator()) HGoto(ret->GetDexPc()));
+      pred->ReplaceSuccessor(exit, new_block);
+      // Since we are removing a predecessor, there's no need to increment `i`.
+    }
+    new_block->AddInstruction(new (graph_->GetAllocator()) HReturnVoid(exit->GetDexPc()));
+  }
+
+  new_block->AddSuccessor(exit);
+  graph_->AddBlock(new_block);
+
+  // Recompute dominance since we added a new block.
+  graph_->ClearDominanceInformation();
+  graph_->ComputeDominanceInformation();
+}
+
 }  // namespace art
