@@ -27,7 +27,7 @@
 
 #include "jni.h"
 
-#include "art_method-inl.h"
+#include "art_method-alloc-inl.h"
 #include "base/mem_map.h"
 #include "base/mutex.h"
 #include "class_linker.h"
@@ -35,7 +35,7 @@
 #include "dex/class_reference.h"
 #include "handle.h"
 #include "handle_scope-inl.h"
-#include "mirror/class-inl.h"
+#include "mirror/class-alloc-inl.h"
 #include "mirror/class_loader.h"
 #include "monitor-inl.h"
 #include "nativehelper/scoped_local_ref.h"
@@ -44,7 +44,7 @@
 #include "scoped_thread_state_change-inl.h"
 #include "thread-inl.h"
 #include "thread_list.h"
-#include "well_known_classes.h"
+#include "well_known_classes-inl.h"
 
 namespace art {
 
@@ -164,40 +164,34 @@ TEST_F(ThreadLifecycleCallbackRuntimeCallbacksTest, ThreadLifecycleCallbackJava)
 
   cb_.state = CallbackState::kBase;  // Ignore main thread attach.
 
-  {
-    ScopedObjectAccess soa(self);
-    MakeExecutable(soa.Decode<mirror::Class>(WellKnownClasses::java_lang_Thread));
-  }
+  ScopedObjectAccess soa(self);
+  MakeExecutable(WellKnownClasses::java_lang_Thread.Get());
 
-  JNIEnv* env = self->GetJniEnv();
+  StackHandleScope<3u> hs(self);
+  Handle<mirror::String> thread_name = hs.NewHandle(
+      mirror::String::AllocFromModifiedUtf8(self, "ThreadLifecycleCallback test thread"));
+  ASSERT_TRUE(thread_name != nullptr);
 
-  ScopedLocalRef<jobject> thread_name(env,
-                                      env->NewStringUTF("ThreadLifecycleCallback test thread"));
-  ASSERT_TRUE(thread_name.get() != nullptr);
+  Handle<mirror::Object> thread_group =
+      hs.NewHandle(soa.Decode<mirror::Object>(runtime_->GetMainThreadGroup()));
+  Handle<mirror::Object> thread =
+      WellKnownClasses::java_lang_Thread_init->NewObject<'L', 'L', 'I', 'Z'>(
+          hs, self, thread_group, thread_name, kMinThreadPriority, /*daemon=*/ false);
+  ASSERT_FALSE(self->IsExceptionPending());
+  ASSERT_TRUE(thread != nullptr);
 
-  ScopedLocalRef<jobject> thread(env, env->AllocObject(WellKnownClasses::java_lang_Thread));
-  ASSERT_TRUE(thread.get() != nullptr);
+  ArtMethod* start_method =
+      thread->GetClass()->FindClassMethod("start", "()V", kRuntimePointerSize);
+  ASSERT_TRUE(start_method != nullptr);
 
-  env->CallNonvirtualVoidMethod(thread.get(),
-                                WellKnownClasses::java_lang_Thread,
-                                WellKnownClasses::java_lang_Thread_init,
-                                runtime_->GetMainThreadGroup(),
-                                thread_name.get(),
-                                kMinThreadPriority,
-                                JNI_FALSE);
-  ASSERT_FALSE(env->ExceptionCheck());
+  start_method->InvokeVirtual<'V'>(self, thread.Get());
+  ASSERT_FALSE(self->IsExceptionPending());
 
-  jmethodID start_id = env->GetMethodID(WellKnownClasses::java_lang_Thread, "start", "()V");
-  ASSERT_TRUE(start_id != nullptr);
+  ArtMethod* join_method = thread->GetClass()->FindClassMethod("join", "()V", kRuntimePointerSize);
+  ASSERT_TRUE(join_method != nullptr);
 
-  env->CallVoidMethod(thread.get(), start_id);
-  ASSERT_FALSE(env->ExceptionCheck());
-
-  jmethodID join_id = env->GetMethodID(WellKnownClasses::java_lang_Thread, "join", "()V");
-  ASSERT_TRUE(join_id != nullptr);
-
-  env->CallVoidMethod(thread.get(), join_id);
-  ASSERT_FALSE(env->ExceptionCheck());
+  join_method->InvokeFinal<'V'>(self, thread.Get());
+  ASSERT_FALSE(self->IsExceptionPending());
 
   EXPECT_EQ(cb_.state, CallbackState::kDied);
 }
@@ -303,6 +297,7 @@ class ClassLoadCallbackRuntimeCallbacksTest : public RuntimeCallbacksTest {
 TEST_F(ClassLoadCallbackRuntimeCallbacksTest, ClassLoadCallback) {
   ScopedObjectAccess soa(Thread::Current());
   jobject jclass_loader = LoadDex("XandY");
+  cb_.data.clear();  // Clear class loading records from `LoadDex()`, if any.
   VariableSizedHandleScope hs(soa.Self());
   Handle<mirror::ClassLoader> class_loader(hs.NewHandle(
       soa.Decode<mirror::ClassLoader>(jclass_loader)));
@@ -514,12 +509,11 @@ TEST_F(MonitorWaitCallbacksTest, WaitUnlocked) {
     ASSERT_TRUE(started);
     {
       ScopedObjectAccess soa(self);
-      cb_.SetInterestingObject(
-          soa.Decode<mirror::Class>(WellKnownClasses::java_util_Collections));
+      cb_.SetInterestingObject(WellKnownClasses::java_util_Collections.Get());
       Monitor::Wait(
           self,
           // Just a random class
-          soa.Decode<mirror::Class>(WellKnownClasses::java_util_Collections),
+          WellKnownClasses::java_util_Collections.Get(),
           /*ms=*/0,
           /*ns=*/0,
           /*interruptShouldThrow=*/false,
