@@ -17,13 +17,13 @@
 import java.lang.reflect.Method;
 
 public class Main {
-  public static void main(String[] args) throws Exception {
-    Class<?> c = Class.forName("IrreducibleLoop");
-    // Note that we don't actually enter the loops in the 'liveness'
-    // method, so this is just a verification that that part of the code we
-    // generated for that method is correct.
-    Method m = c.getMethod("liveness", int.class);
-    Object[] arguments = { 42 };
-    System.out.println(m.invoke(null, arguments));
-  }
+    public static void main(String[] args) throws Exception {
+        Class<?> c = Class.forName("IrreducibleLoop");
+        // Note that we don't actually enter the loops in the 'liveness'
+        // method, so this is just a verification that that part of the code we
+        // generated for that method is correct.
+        Method m = c.getMethod("liveness", int.class, int.class);
+        Object[] arguments = {42, 12};
+        System.out.println(m.invoke(null, arguments));
+    }
 }
