@@ -24,9 +24,10 @@
 #include "base/casts.h"
 #include "base/mutex-inl.h"
 #include "base/time_utils.h"
+#include "indirect_reference_table.h"
 #include "jni/jni_env_ext.h"
 #include "managed_stack-inl.h"
-#include "obj_ptr.h"
+#include "obj_ptr-inl.h"
 #include "suspend_reason.h"
 #include "thread-current-inl.h"
 #include "thread_pool.h"
@@ -34,11 +35,37 @@
 namespace art {
 
 // Quickly access the current thread from a JNIEnv.
-static inline Thread* ThreadForEnv(JNIEnv* env) {
+inline Thread* Thread::ForEnv(JNIEnv* env) {
   JNIEnvExt* full_env(down_cast<JNIEnvExt*>(env));
   return full_env->GetSelf();
 }
 
+inline ObjPtr<mirror::Object> Thread::DecodeJObject(jobject obj) const {
+  if (obj == nullptr) {
+    return nullptr;
+  }
+  IndirectRef ref = reinterpret_cast<IndirectRef>(obj);
+  if (LIKELY(IndirectReferenceTable::IsJniTransitionOrLocalReference(ref))) {
+    // For JNI transitions, the `jclass` for a static method points to the
+    // `CompressedReference<>` in the `ArtMethod::declaring_class_` and other `jobject`
+    // arguments point to spilled stack references but a `StackReference<>` is just
+    // a subclass of `CompressedReference<>`. Local references also point to
+    // a `CompressedReference<>` encapsulated in a `GcRoot<>`.
+    if (kIsDebugBuild && IndirectReferenceTable::GetIndirectRefKind(ref) == kJniTransition) {
+      CHECK(IsJniTransitionReference(obj));
+    }
+    auto* cref = IndirectReferenceTable::ClearIndirectRefKind<
+        mirror::CompressedReference<mirror::Object>*>(ref);
+    ObjPtr<mirror::Object> result = cref->AsMirrorPtr();
+    if (kIsDebugBuild && IndirectReferenceTable::GetIndirectRefKind(ref) != kJniTransition) {
+      CHECK_EQ(result, tlsPtr_.jni_env->locals_.Get(ref));
+    }
+    return result;
+  } else {
+    return DecodeGlobalJObject(obj);
+  }
+}
+
 inline void Thread::AllowThreadSuspension() {
   CheckSuspend();
   // Invalidate the current thread's object pointers (ObjPtr) to catch possible moving GC bugs due
@@ -373,7 +400,7 @@ inline bool Thread::PushOnThreadLocalAllocationStack(mirror::Object* obj) {
 }
 
 inline bool Thread::GetWeakRefAccessEnabled() const {
-  CHECK(kUseReadBarrier);
+  DCHECK(gUseReadBarrier);
   DCHECK(this == Thread::Current());
   WeakRefAccessState s = tls32_.weak_ref_access_enabled.load(std::memory_order_relaxed);
   if (LIKELY(s == WeakRefAccessState::kVisiblyEnabled)) {
@@ -428,7 +455,8 @@ inline bool Thread::ModifySuspendCount(Thread* self,
                                        int delta,
                                        AtomicInteger* suspend_barrier,
                                        SuspendReason reason) {
-  if (delta > 0 && ((kUseReadBarrier && this != self) || suspend_barrier != nullptr)) {
+  if (delta > 0 &&
+      (((gUseUserfaultfd || gUseReadBarrier) && this != self) || suspend_barrier != nullptr)) {
     // When delta > 0 (requesting a suspend), ModifySuspendCountInternal() may fail either if
     // active_suspend_barriers is full or we are in the middle of a thread flip. Retry in a loop.
     while (true) {
