@@ -18,11 +18,12 @@
 
 #include "base/casts.h"
 #include "entrypoints/quick/quick_entrypoints.h"
+#include "indirect_reference_table.h"
 #include "lock_word.h"
 #include "thread.h"
 #include "utils/assembler.h"
 
-namespace art {
+namespace art HIDDEN {
 namespace x86 {
 
 static Register GetScratchRegister() {
@@ -165,36 +166,24 @@ void X86JNIMacroAssembler::Store(ManagedRegister mbase,
   }
 }
 
-void X86JNIMacroAssembler::StoreRef(FrameOffset dest, ManagedRegister msrc) {
-  X86ManagedRegister src = msrc.AsX86();
-  CHECK(src.IsCpuRegister());
-  __ movl(Address(ESP, dest), src.AsCpuRegister());
-}
-
 void X86JNIMacroAssembler::StoreRawPtr(FrameOffset dest, ManagedRegister msrc) {
   X86ManagedRegister src = msrc.AsX86();
   CHECK(src.IsCpuRegister());
   __ movl(Address(ESP, dest), src.AsCpuRegister());
 }
 
-void X86JNIMacroAssembler::StoreImmediateToFrame(FrameOffset dest, uint32_t imm) {
-  __ movl(Address(ESP, dest), Immediate(imm));
-}
-
-void X86JNIMacroAssembler::StoreStackOffsetToThread(ThreadOffset32 thr_offs, FrameOffset fr_offs) {
-  Register scratch = GetScratchRegister();
-  __ leal(scratch, Address(ESP, fr_offs));
-  __ fs()->movl(Address::Absolute(thr_offs), scratch);
-}
-
-void X86JNIMacroAssembler::StoreStackPointerToThread(ThreadOffset32 thr_offs) {
-  __ fs()->movl(Address::Absolute(thr_offs), ESP);
-}
-
-void X86JNIMacroAssembler::StoreSpanning(FrameOffset /*dst*/,
-                                         ManagedRegister /*src*/,
-                                         FrameOffset /*in_off*/) {
-  UNIMPLEMENTED(FATAL);  // this case only currently exists for ARM
+void X86JNIMacroAssembler::StoreStackPointerToThread(ThreadOffset32 thr_offs, bool tag_sp) {
+  if (tag_sp) {
+    // There is no free register, store contents onto stack and restore back later.
+    Register scratch = ECX;
+    __ movl(Address(ESP, -32), scratch);
+    __ movl(scratch, ESP);
+    __ orl(scratch, Immediate(0x2));
+    __ fs()->movl(Address::Absolute(thr_offs), scratch);
+    __ movl(scratch, Address(ESP, -32));
+  } else {
+    __ fs()->movl(Address::Absolute(thr_offs), ESP);
+  }
 }
 
 void X86JNIMacroAssembler::Load(ManagedRegister mdest, FrameOffset src, size_t size) {
@@ -233,61 +222,6 @@ void X86JNIMacroAssembler::Load(ManagedRegister mdest,
   }
 }
 
-void X86JNIMacroAssembler::LoadFromThread(ManagedRegister mdest, ThreadOffset32 src, size_t size) {
-  X86ManagedRegister dest = mdest.AsX86();
-  if (dest.IsNoRegister()) {
-    CHECK_EQ(0u, size);
-  } else if (dest.IsCpuRegister()) {
-    if (size == 1u) {
-      __ fs()->movzxb(dest.AsCpuRegister(), Address::Absolute(src));
-    } else {
-      CHECK_EQ(4u, size);
-      __ fs()->movl(dest.AsCpuRegister(), Address::Absolute(src));
-    }
-  } else if (dest.IsRegisterPair()) {
-    CHECK_EQ(8u, size);
-    __ fs()->movl(dest.AsRegisterPairLow(), Address::Absolute(src));
-    __ fs()->movl(dest.AsRegisterPairHigh(), Address::Absolute(ThreadOffset32(src.Int32Value()+4)));
-  } else if (dest.IsX87Register()) {
-    if (size == 4) {
-      __ fs()->flds(Address::Absolute(src));
-    } else {
-      __ fs()->fldl(Address::Absolute(src));
-    }
-  } else {
-    CHECK(dest.IsXmmRegister());
-    if (size == 4) {
-      __ fs()->movss(dest.AsXmmRegister(), Address::Absolute(src));
-    } else {
-      __ fs()->movsd(dest.AsXmmRegister(), Address::Absolute(src));
-    }
-  }
-}
-
-void X86JNIMacroAssembler::LoadRef(ManagedRegister mdest, FrameOffset src) {
-  X86ManagedRegister dest = mdest.AsX86();
-  CHECK(dest.IsCpuRegister());
-  __ movl(dest.AsCpuRegister(), Address(ESP, src));
-}
-
-void X86JNIMacroAssembler::LoadRef(ManagedRegister mdest, ManagedRegister base, MemberOffset offs,
-                           bool unpoison_reference) {
-  X86ManagedRegister dest = mdest.AsX86();
-  CHECK(dest.IsCpuRegister() && dest.IsCpuRegister());
-  __ movl(dest.AsCpuRegister(), Address(base.AsX86().AsCpuRegister(), offs));
-  if (unpoison_reference) {
-    __ MaybeUnpoisonHeapReference(dest.AsCpuRegister());
-  }
-}
-
-void X86JNIMacroAssembler::LoadRawPtr(ManagedRegister mdest,
-                                      ManagedRegister base,
-                                      Offset offs) {
-  X86ManagedRegister dest = mdest.AsX86();
-  CHECK(dest.IsCpuRegister() && dest.IsCpuRegister());
-  __ movl(dest.AsCpuRegister(), Address(base.AsX86().AsCpuRegister(), offs));
-}
-
 void X86JNIMacroAssembler::LoadRawPtrFromThread(ManagedRegister mdest, ThreadOffset32 offs) {
   X86ManagedRegister dest = mdest.AsX86();
   CHECK(dest.IsCpuRegister());
@@ -402,37 +336,9 @@ void X86JNIMacroAssembler::Move(ManagedRegister mdest, ManagedRegister msrc, siz
   }
 }
 
-void X86JNIMacroAssembler::CopyRef(FrameOffset dest, FrameOffset src) {
-  Register scratch = GetScratchRegister();
-  __ movl(scratch, Address(ESP, src));
-  __ movl(Address(ESP, dest), scratch);
-}
-
-void X86JNIMacroAssembler::CopyRef(FrameOffset dest,
-                                   ManagedRegister base,
-                                   MemberOffset offs,
-                                   bool unpoison_reference) {
-  Register scratch = GetScratchRegister();
-  __ movl(scratch, Address(base.AsX86().AsCpuRegister(), offs));
-  if (unpoison_reference) {
-    __ MaybeUnpoisonHeapReference(scratch);
-  }
-  __ movl(Address(ESP, dest), scratch);
-}
-
-void X86JNIMacroAssembler::CopyRawPtrFromThread(FrameOffset fr_offs, ThreadOffset32 thr_offs) {
-  Register scratch = GetScratchRegister();
-  __ fs()->movl(scratch, Address::Absolute(thr_offs));
-  __ movl(Address(ESP, fr_offs), scratch);
-}
-
-void X86JNIMacroAssembler::CopyRawPtrToThread(ThreadOffset32 thr_offs,
-                                              FrameOffset fr_offs,
-                                              ManagedRegister mscratch) {
-  X86ManagedRegister scratch = mscratch.AsX86();
-  CHECK(scratch.IsCpuRegister());
-  Load(scratch, fr_offs, 4);
-  __ fs()->movl(Address::Absolute(thr_offs), scratch.AsCpuRegister());
+void X86JNIMacroAssembler::Move(ManagedRegister mdest, size_t value) {
+  X86ManagedRegister dest = mdest.AsX86();
+  __ movl(dest.AsCpuRegister(), Immediate(value));
 }
 
 void X86JNIMacroAssembler::Copy(FrameOffset dest, FrameOffset src, size_t size) {
@@ -446,67 +352,6 @@ void X86JNIMacroAssembler::Copy(FrameOffset dest, FrameOffset src, size_t size)
   }
 }
 
-void X86JNIMacroAssembler::Copy(FrameOffset /*dst*/,
-                                ManagedRegister /*src_base*/,
-                                Offset /*src_offset*/,
-                                ManagedRegister /*scratch*/,
-                                size_t /*size*/) {
-  UNIMPLEMENTED(FATAL);
-}
-
-void X86JNIMacroAssembler::Copy(ManagedRegister dest_base,
-                                Offset dest_offset,
-                                FrameOffset src,
-                                ManagedRegister scratch,
-                                size_t size) {
-  CHECK(scratch.IsNoRegister());
-  CHECK_EQ(size, 4u);
-  __ pushl(Address(ESP, src));
-  __ popl(Address(dest_base.AsX86().AsCpuRegister(), dest_offset));
-}
-
-void X86JNIMacroAssembler::Copy(FrameOffset dest,
-                                FrameOffset src_base,
-                                Offset src_offset,
-                                ManagedRegister mscratch,
-                                size_t size) {
-  Register scratch = mscratch.AsX86().AsCpuRegister();
-  CHECK_EQ(size, 4u);
-  __ movl(scratch, Address(ESP, src_base));
-  __ movl(scratch, Address(scratch, src_offset));
-  __ movl(Address(ESP, dest), scratch);
-}
-
-void X86JNIMacroAssembler::Copy(ManagedRegister dest,
-                                Offset dest_offset,
-                                ManagedRegister src,
-                                Offset src_offset,
-                                ManagedRegister scratch,
-                                size_t size) {
-  CHECK_EQ(size, 4u);
-  CHECK(scratch.IsNoRegister());
-  __ pushl(Address(src.AsX86().AsCpuRegister(), src_offset));
-  __ popl(Address(dest.AsX86().AsCpuRegister(), dest_offset));
-}
-
-void X86JNIMacroAssembler::Copy(FrameOffset dest,
-                                Offset dest_offset,
-                                FrameOffset src,
-                                Offset src_offset,
-                                ManagedRegister mscratch,
-                                size_t size) {
-  Register scratch = mscratch.AsX86().AsCpuRegister();
-  CHECK_EQ(size, 4u);
-  CHECK_EQ(dest.Int32Value(), src.Int32Value());
-  __ movl(scratch, Address(ESP, src));
-  __ pushl(Address(scratch, src_offset));
-  __ popl(Address(scratch, dest_offset));
-}
-
-void X86JNIMacroAssembler::MemoryBarrier(ManagedRegister) {
-  __ mfence();
-}
-
 void X86JNIMacroAssembler::CreateJObject(ManagedRegister mout_reg,
                                          FrameOffset spilled_reference_offset,
                                          ManagedRegister min_reg,
@@ -547,6 +392,20 @@ void X86JNIMacroAssembler::CreateJObject(FrameOffset out_off,
   __ movl(Address(ESP, out_off), scratch);
 }
 
+void X86JNIMacroAssembler::DecodeJNITransitionOrLocalJObject(ManagedRegister reg,
+                                                             JNIMacroLabel* slow_path,
+                                                             JNIMacroLabel* resume) {
+  constexpr uint32_t kGlobalOrWeakGlobalMask =
+      dchecked_integral_cast<uint32_t>(IndirectReferenceTable::GetGlobalOrWeakGlobalMask());
+  constexpr uint32_t kIndirectRefKindMask =
+      dchecked_integral_cast<uint32_t>(IndirectReferenceTable::GetIndirectRefKindMask());
+  __ testl(reg.AsX86().AsCpuRegister(), Immediate(kGlobalOrWeakGlobalMask));
+  __ j(kNotZero, X86JNIMacroLabel::Cast(slow_path)->AsX86());
+  __ andl(reg.AsX86().AsCpuRegister(), Immediate(~kIndirectRefKindMask));
+  __ j(kZero, X86JNIMacroLabel::Cast(resume)->AsX86());  // Skip load for null.
+  __ movl(reg.AsX86().AsCpuRegister(), Address(reg.AsX86().AsCpuRegister(), /*disp=*/ 0));
+}
+
 void X86JNIMacroAssembler::VerifyObject(ManagedRegister /*src*/, bool /*could_be_null*/) {
   // TODO: not validating references
 }
@@ -724,6 +583,12 @@ void X86JNIMacroAssembler::TestMarkBit(ManagedRegister mref,
   __ j(UnaryConditionToX86Condition(cond), X86JNIMacroLabel::Cast(label)->AsX86());
 }
 
+
+void X86JNIMacroAssembler::TestByteAndJumpIfNotZero(uintptr_t address, JNIMacroLabel* label) {
+  __ cmpb(Address::Absolute(address), Immediate(0));
+  __ j(kNotZero, X86JNIMacroLabel::Cast(label)->AsX86());
+}
+
 void X86JNIMacroAssembler::Bind(JNIMacroLabel* label) {
   CHECK(label != nullptr);
   __ Bind(X86JNIMacroLabel::Cast(label)->AsX86());
